
clear all;
close all;
format long e;
  
% random seed is the clock, clock is used because the time we start something is relatively random
RandSeed=clock;          
rand('seed',RandSeed(5));  % seed is the first number to start the generator

Nt = 128;
HS = 2000
HH = 50
QC = 2500
nsim = 50000;      % number of simulations
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 

%****** prior values (c) and parameter ranges (cmin, cmax) ******************
%        f1            f2              parameter names 
a    =  [0.05         0.55]';   
amin =  [0.049         0.54];    
amax =  [0.051         0.56];
%        k1         k2          k3
b    =  [0.010     0.00600     0.00020]';
bmin =  [0.001     0.00100     0.000019];
bmax =  [0.01      0.00210     0.000021];
%       f2,1         f3,1         f1,2       f3,2        f1,3
c    =  [0.45       0.006         0.1        0.0031        0.2]';
cmin =  [0.1        0.015         0.09        0.029        0.0]; 
cmax =  [0.6        0.020         0.11        0.031        0.5];
%      Q10(p1)      Q10(p2)       Q10(p3)      
d    =  [2          3            3]';   
dmin =  [0            0            0];    
dmax =  [3.5          6            6];
B    =  [1            0            0];
%*********** read in data ***************
data = site2(Nt);
day = data(:,1);              % assign columns to values    
flux  = data(:,2);            % unit : Mg ha-1 yr-1
ctotal = data(:,3);           % Mg ha-1
tau = data(:,4);              % yr

stdev2 = 4.864232465;%% flux for site2
stdev1 = 21.81889229;%% ctotal for site2

%simulation starts
adif  = (amax-amin)';
bdif  = (bmax-bmin)';
cdif = (cmax-cmin)';
ddif = (dmax-dmin)';
 
a_op=amin'+rand*adif;
a_new=zeros(3,1);      % create a new array with zeros  
b_op=bmin'+rand*bdif;
J_last = 30000000;
b_new=zeros(3,1);      % create a new array with zeros  
record_index=1;
c_op=cmin'+rand*cdif;
c_new=zeros(5,1);
d_op=dmin'+rand*ddif;
d_new=zeros(3,1);
DJ1=2*stdev1^2;
DJ2=2*stdev2^2;
%Simulation starts
upgraded=0;

for simu=1:nsim
    counter=simu
    upgraded % shows the upgraded number
    %%%%%%%  %generate a new point
    
    a_new=Generate_a (a_op,amin, amax);
    b_new=Generate_b (b_op,bmin, bmax);
    c_new=Generate_c (c_op,cmin, cmax);
    d_new=Generate_d (d_op,dmin, dmax);
    
    f1 = a(1);          %fraction of inital total carbon to Pool 1
    f2 = a(2);          %fraction of inital total carbon to Pool 2
    f3 = 1-f1-f2;       %fraction of inital total carbon to Pool 3
    k1 = b(1);
    k2 = b(2);
    k3 = b(3);
    f21 = c(1);         %fraction from pool 1 to pool 2
    f31 = c(2);         %fraction from pool 1 to pool 3 
    f12 = c(3);         %fraction from pool 2 to pool 1
    f32 = c(4);         %fraction from pool 2 to pool 3
    f13 = c(5);         %fraction from pool 3 to pool 1
    Q1 = d_new(1);   
    Q2 = d_new(2);
    Q3 = d_new(3);
   
    T1P1 = zeros(day(Nt),1);
    T1P2 = zeros(day(Nt),1);
    T1P3 = zeros(day(Nt),1);
    T1R1 = zeros(day(Nt),1);
    T1R2 = zeros(day(Nt),1);
    T1R3 = zeros(day(Nt),1);
    T1C1 = zeros(day(Nt),1);
    T1C2 = zeros(day(Nt),1);
    T1C3 = zeros(day(Nt),1);
    T1fR1 = zeros(day(Nt),1);
    T1fR2 = zeros(day(Nt),1);
    T1fR3 = zeros(day(Nt),1);
    resp_mod1 = zeros(day(Nt),1);
    resp_mod2 = zeros(day(Nt),1);
    
    T1A1 = zeros(day(Nt),1);
    T1A2 = zeros(day(Nt),1);
    T1A3 = zeros(day(Nt),1);
    TransitP1 = zeros(day(Nt),1);
    TransitP2 = zeros(day(Nt),1);
    TransitP3 = zeros(day(Nt),1);
     
     T1A  = zeros(3,1);
     T1P  = zeros(3,1);
     F    = zeros(3,3);
     K    = zeros(3,3);
     RR   = zeros(3,3);

  for  i = 1;
       T1P1(i) = flux(i)/(k1*Q1*(1-f21*f12-f31*f13+f21*f32*f13));
       T1P2(i) = f21*k1*Q1*T1P1(i)/k2*Q2;
       T1P3(i) = f31*k1*Q1*T1P1(i)/k3*Q3+f32*k2*Q2*T1P2(i)/k3*Q3;
       
       T1R1(i) = (1-f21-f31)*k1*Q1*f1*ctotal(i);       %respiration rate of pool1 
       T1R2(i) = (1-f12-f32)*k2*Q2*f2*ctotal(i);       %respiration rate of pool2 
       T1R3(i) = (1-f13)*k3*Q3*f3*ctotal(i);           %respiration rate of pool3
       
       T1C1(i) = T1R1(i);
       T1C2(i) = T1R2(i);
       T1C3(i) = T1R3(i);
       
       T1fR1(i) = T1R1(i)/(T1R1(i)+T1R2(i)+T1R3(i));   % proportion of R1 in Rtotal
       T1fR2(i) = T1R2(i)/(T1R1(i)+T1R2(i)+T1R3(i));   % proportion of R2 in Rtotal
       T1fR3(i) = T1R3(i)/(T1R1(i)+T1R2(i)+T1R3(i));   % proportion of R3 in Rtotal

     T1P = [T1P1(i);T1P2(i);T1P3(i)];
     F(1,2) = f12;
     F(1,3) = f13;
     F(2,1) = f21;
     F(3,1) = f31;
     F(3,2) = f32;
     F(1,1) = -1;
     F(2,2) = -1;
     F(3,3) = -1;
     K(1,1) = k1*Q1*T1P1(i);
     K(2,2) = k2*Q2*T1P2(i);
     K(3,3) = k3*Q3*T1P3(i);
     FK     = F*K;
     
     T1A = -(FK)^-1*T1P;
     m=1/T1P1(i);
     n=1;      
     T1A1(i) = T1A(1,1);
     T1A2(i) = T1A(2,1);
     T1A3(i) = T1A(3,1);
      
     TransitP1(i)=(T1A1(i)*T1P1(i)*(1-f21-f31)*k1*Q1)/(T1P1(i)*(1-f21-f31)*k1*Q1+T1P2(i)*(1-f32)*k2*Q2+T1P3(i)*k3*Q3);
     TransitP2(i)=(T1A2(i)*T1P2(i)*(1-f32)*k2*Q2)/(T1P1(i)*(1-f21-f31)*k1*Q1+T1P2(i)*(1-f32)*k2*Q2+T1P3(i)*k3*Q3);
     TransitP3(i)=(T1A3(i)*T1P3(i)*k3*Q3)/(T1P1(i)*(1-f21-f31)*k1*Q1+T1P2(i)*(1-f32)*k2*Q2+T1P3(i)*k3*Q3);

      resp_mod1(i) = T1P1(i)+T1P2(i)+T1P3(i);
      resp_mod2(i) = T1R1(i)+T1R2(i)+T1R3(i);
  end       
   
    for i = 2:day(Nt)
        
       T1P1(i) = T1P1(i-1)-k1*Q1*T1P1(i-1)+f12*k2*Q2*T1P2(i-1)+ f13*k3*Q3*T1P3(i-1)+flux(i-1);
       T1P2(i) = T1P2(i-1)-k2*Q2*T1P2(i-1)+f21*k1*Q1*T1P1(i-1);
       T1P3(i) = T1P3(i-1)-k3*Q3*T1P3(i-1)+f31*k1*Q1*T1P1(i-1)+f32*k2*Q2*T1P2(i-1);
       %age structure
       T1A1(i) = T1A1(i-1)+1.0 + ((T1A2(i-1) - T1A1(i-1))*f12*k2*Q2*T1P2(i-1) + (T1A3(i-1) - T1A1(i-1))*f13*k3*Q3*T1P3(i-1)-T1A1(i-1))/T1P1(i-1);
       T1A2(i) = T1A2(i-1)+1.0 + ((T1A1(i-1) - T1A2(i-1))*f21*k1*Q1*T1P1(i-1))/T1P2(i-1);
       T1A3(i) = T1A3(i-1)+1.0 + ((T1A1(i-1) - T1A3(i-1))*f31*k1*Q1*T1P1(i-1) + (T1A2(i-1) - T1A3(i-1))*f32*k2*Q2*T1P2(i-1))/T1P3(i-1);
       %transit time
       TransitP1(i)=(T1A1(i)*T1P1(i)*(1-f21-f31)*k1*Q1)/(T1P1(i)*(1-f21-f31)*k1*Q1+T1P2(i)*(1-f32)*k2*Q2+T1P3(i)*k3*Q3);
       TransitP2(i)=(T1A2(i)*T1P2(i)*(1-f32)*k2*Q2)/(T1P1(i)*(1-f21-f31)*k1*Q1+T1P2(i)*(1-f32)*k2*Q2+T1P3(i)*k3*Q3);
       TransitP3(i)=(T1A3(i)*T1P3(i)*k3*Q3)/(T1P1(i)*(1-f21-f31)*k1*Q1+T1P2(i)*(1-f32)*k2*Q2+T1P3(i)*k3*Q3);
       
       T1meanA = (T1A1(i)*T1P1(i)+T1A2(i)*T1P2(i)+T1A3(i)*T1P3(i))/(T1P1(i)+T1P2(i)+T1P3(i));
       T1Ptotal = T1P1(i)+T1P2(i)+T1P3(i);
       Transit = (T1A1(i)*T1P1(i)*(1-f21-f31)*k1*Q1+T1A2(i)*T1P2(i)*(1-f32)*k2*Q2+T1A3(i)*T1P3(i)*k3*Q3)/(T1P1(i)*(1-f21-f31)*k1*Q1+T1P2(i)*(1-f32)*k2*Q2+T1P3(i)*k3*Q3);
       %respiration
       T1R1(i) = (1-f21-f31)*k1*Q1*T1P1(i-1);       %respiration rate of pool1 
       T1R2(i) = (1-f12-f32)*k2*Q2*T1P2(i-1);       %respiration rate of pool2 
       T1R3(i) = (1-f13)*k3*Q3*T1P3(i-1);           %respiration rate of pool3
       
       T1C1(i) = T1C1(i-1)+T1R1(i);               %cummulative carbon respired from pool1
       T1C2(i) = T1C2(i-1)+T1R2(i);               %cummulative carbon respired from pool2
       T1C3(i) = T1C3(i-1)+T1R3(i);               %cummulative carbon respired from pool3

       T1fR1(i) = T1R1(i)/(T1R1(i)+T1R2(i)+T1R3(i));   % proportion of R1 in Rtotal
       T1fR2(i) = T1R2(i)/(T1R1(i)+T1R2(i)+T1R3(i));   % proportion of R2 in Rtotal
       T1fR3(i) = T1R3(i)/(T1R1(i)+T1R2(i)+T1R3(i));   % proportion of R3 in Rtotal
       
       resp_mod1(i) = T1P1(i)+T1P2(i)+T1P3(i);
       resp_mod2(i) = T1R1(i)+T1R2(i)+T1R3(i);
    end
    
    T1p1 = zeros(Nt,1);
    T1p2 = zeros(Nt,1);
    T1p3 = zeros(Nt,1);
    T1r1 = zeros(Nt,1);
    T1r2 = zeros(Nt,1);
    T1r3 = zeros(Nt,1);
    T1c1 = zeros(Nt,1);
    T1c2 = zeros(Nt,1);
    T1c3 = zeros(Nt,1);
    T1fr1 = zeros(Nt,1);
    T1fr2 = zeros(Nt,1);
    T1fr3 = zeros(Nt,1);
    CC_mod1 = zeros(Nt,1);
    CC_mod2 = zeros(Nt,1);
    for m = 1:Nt
        i = day(m);
        
        T1p1(m)=T1P1(i);
        T1p2(m)=T1P2(i);
        T1p3(m)=T1P3(i);
        
        T1r1(m)=T1R1(i);
        T1r2(m)=T1R2(i);
        T1r3(m)=T1R3(i);
        
        T1c1(m)=T1C1(i);
        T1c2(m)=T1C2(i);
        T1c3(m)=T1C3(i);
        
        T1fr1(m)=T1fR1(i);
        T1fr2(m)=T1fR2(i);
        T1fr3(m)=T1fR3(i);
        
        CC_mod1(m) = resp_mod1(i);
        CC_mod2(m) = resp_mod2(i);
    end
    
    % calculate cost function 
    J1  = (norm(CC_mod1' - ctotal'))^2;
    J2  = (norm(CC_mod2' - flux'))^2;    % data model error
    J_new = J1/DJ1+J2/DJ2;
    delta_J = J_new -J_last;
	
	if min(1, exp(-delta_J)) >rand
        a_op=a_new;
        b_op=b_new;
        c_op=c_new;
        d_op=d_new;
        J_last=J_new;
        upgraded=upgraded+1;
        a_upgraded(:,upgraded)=a_op;  
        b_upgraded(:,upgraded)=b_op;  
        c_upgraded(:,upgraded)=c_op;
        d_upgraded(:,upgraded)=d_op;
        
        T1CC_record(:,upgraded)=CC_mod1;
        T1P1_record(:,upgraded)=T1p1;
        T1P2_record(:,upgraded)=T1p2;
        T1P3_record(:,upgraded)=T1p3;
        T1R1_record(:,upgraded)=T1r1;
        T1R2_record(:,upgraded)=T1r2;
        T1R3_record(:,upgraded)=T1r3;
        T1C1_record(:,upgraded)=T1c1;
        T1C2_record(:,upgraded)=T1c2;
        T1C3_record(:,upgraded)=T1c3;
        T1fR1_record(:,upgraded)=T1fr1;
        T1fR2_record(:,upgraded)=T1fr2;
        T1fR3_record(:,upgraded)=T1fr3;
        J_record(:,upgraded) = J_last;
    end
    
end



